/*
 * filebrowser.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <KStandardDirs>

#include "filebrowser.h"



FileBrowser::FileBrowser( OsdConfig *conf, VideoWidget *parent ) : OsdAnim(  conf )
{
	vw = parent;

	connect( this, SIGNAL(play(QString)), vw, SLOT(playMrl(QString)) );

	QString def = KStandardDirs::locate("data", "kaffeinegl/defthumb.jpg" );
	defThumb.load( def );

	list = new OsdList( conf );
	connect( list, SIGNAL(wantRepaint()), this, SIGNAL(wantRepaint()) );

	thumbView = new OsdThumbView( conf );
	connect( thumbView, SIGNAL(clicked(QString)), this, SLOT(thumbClicked(QString)) );
	connect( thumbView, SIGNAL(wantRepaint()), this, SIGNAL(wantRepaint()) );
	connect( thumbView, SIGNAL(startRepaint()), vw, SLOT(startRepaint()) );
	connect( thumbView, SIGNAL(stopRepaint()), vw, SLOT(stopRepaint()) );

	dataPath = KStandardDirs::locateLocal("data", "kaffeinegl/thumbs/" );
	bthread.setThumbDir( dataPath );
	connect( &bthread, SIGNAL(done()), this, SLOT(updateList()), Qt::BlockingQueuedConnection );

	currentDir.setPath( QDir::homePath() );
	itemClicked( "" );
}

FileBrowser::~FileBrowser()
{
	delete thumbView;
	delete list;
}

void FileBrowser::updateThumb( const QString &s )
{
	thumbView->updateThumb( dataPath+s );
}

void FileBrowser::thumbClicked( QString text )
{
	kDebug() << text;
	emit play( currentDir.path()+"/"+text );
}

void FileBrowser::itemClicked( QString text )
{
	if ( text=="Up.." ) {
		if ( !currentDir.cdUp() )
			return;
	}
	else {
		currentDir.cd( text );
	}
	list->clear();
	thumbView->clear();
	emit wantRepaint();
	bthread.browse( &currentDir, &dirList, &filesList, &thumbsList );
}

void FileBrowser::updateList()
{
	int i;
	OsdListItem *it;
	OsdThumbItem *thumb;
	QString s, t;

	if ( thumbsList.count() ) {
		vw->thumb( thumbsList );
	}

	it = new OsdListItem( config, "Up.." );
	connect( it, SIGNAL(clicked(QString)), this, SLOT(itemClicked(QString)) );
	list->appendEntry( it );
	for ( i=0; i<dirList.count(); ++i ) {
		if ( dirList.at(i)=="." || dirList.at(i)=="..")
			continue;
		it = new OsdListItem( config, dirList.at(i) );
		connect( it, SIGNAL(clicked(QString)), this, SLOT(itemClicked(QString)) );
		list->appendEntry( it );
	}
	for ( i=0; i<filesList.count(); ++i ) {
		s = filesList.at(i);
		t = QDir( dataPath ).filePath( QString(currentDir.filePath(s).remove("/").append(".jpg")));
		thumb = new OsdThumbItem( config, s, t, &defThumb );
		thumbView->append( thumb );
	}
	list->resize( list->width(), list->height() );
	thumbView->resize( thumbView->width(), thumbView->height() );
	emit wantRepaint();
}

void FileBrowser::resize( int w, int h )
{
	rect.setWidth( w );
	rect.setHeight( h );
	list->setGeometry( rect.x(), rect.y(), 250, h );
	list->resize( 250, h );
	thumbView->setGeometry( rect.x()+list->width(), rect.y(), rect.width()-list->width(), rect.height() );
	thumbView->resize( thumbView->width(), thumbView->height() );
}

void FileBrowser::mouseMove( QPoint &p )
{
	if ( isHidden() || isMoving() )
		return;

	list->mouseMove( p );
}

bool FileBrowser::mouseClicked( QMouseEvent *e )
{
	if ( isHidden() || isMoving() )
		return false;

	if ( list->mouseClicked( e ) )
		return true;
	if ( thumbView->mouseClicked( e ) )
		return true;
	return false;
}

void FileBrowser::mouseReleased( QMouseEvent *e )
{
	list ->mouseReleased( e );
}

void FileBrowser::wheelEvent( QWheelEvent *e )
{
	if ( isHidden() || isMoving() )
		return;

	list->wheelEvent( e );
	thumbView->wheelEvent( e );
}

void FileBrowser::paint()
{
	if ( isHidden() && !isMoving() )
		return;

	if ( rect.height()<0 )
		return;

	setTransform();

	config->func->drawQuad( rect, config->backgroundColor() );

	list->paint();
	thumbView->paint();

	resetTransform();
}
