/*
 * mgl.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "mgl.h"
#include <GL/glx.h>



GLFunc::GLFunc( const QGLContext *ct )
{
	context = ct;
}



void GLFunc::init()
{
	QString extensions = (const char*)glGetString(GL_EXTENSIONS);

	mglActiveTexture = (GLACTIVETEXTUREARB)context->getProcAddress( "glActiveTexture" );

	if ( extensions.contains("GL_ARB_fragment_program") ) {
		mglGenPrograms = (GLGENPROGRAMSARB)context->getProcAddress( "glGenProgramsARB" );
		mglBindProgram = (GLBINDPROGRAMARB)context->getProcAddress( "glBindProgramARB" );
		mglProgramString = (GLPROGRAMSTRINGARB)context->getProcAddress( "glProgramStringARB" );
		hasFragProg = mglGenPrograms && mglBindProgram && mglProgramString;
	}
	else
		hasFragProg = false;

	if ( extensions.contains("GL_ARB_texture_rectangle") && extensions.contains("texture_non_power_of_two") ) {
		defaultTarget = GL_TEXTURE_2D;
		hasTextureRect = true;
	}
	else {
		defaultTarget = GL_TEXTURE_2D;
		hasTextureRect = false;
	}

	if ( extensions.contains("GL_ARB_fragment_shader") && hasTextureRect ) {
		mglCreateProgramObject = (GLCREATEPROGRAMOBJECTARB)context->getProcAddress( "glCreateProgramObjectARB" );
		mglCreateShaderObject = (GLCREATESHADEROBJECTARB)context->getProcAddress( "glCreateShaderObjectARB" );
		mglShaderSource = (GLSHADERSOURCEARB)context->getProcAddress( "glShaderSourceARB" );
		mglCompileShader = (GLCOMPILESHADERARB)context->getProcAddress( "glCompileShaderARB" );
		mglGetObjectParameteriv = (GLGETOBJECTPARAMETERIVARB)context->getProcAddress( "glGetObjectParameterivARB" );
		mglGetInfoLog = (GLGETINFOLOGARB)context->getProcAddress( "glGetInfoLogARB" );
		mglAttachObject = (GLATTACHOBJECTARB)context->getProcAddress( "glAttachObjectARB" );
		mglLinkProgram = (GLLINKPROGRAMARB)context->getProcAddress( "glLinkProgramARB" );
		mglUseProgramObject = (GLUSEPROGRAMOBJECTARB)context->getProcAddress( "glUseProgramObjectARB" );
		mglGetUniformLocation = (GLGETUNIFORMLOCATIONARB)context->getProcAddress( "glGetUniformLocationARB" );
		mglUniform1i = (GLUNIFORM1IARB)context->getProcAddress( "glUniform1iARB" );
		hasFragShader = mglCreateProgramObject && mglCreateShaderObject && mglShaderSource && mglCompileShader && mglGetObjectParameteriv && mglGetInfoLog && mglAttachObject && mglLinkProgram && mglUseProgramObject && mglGetUniformLocation && mglUniform1i;
	}
	else
		hasFragShader = false;

	if ( extensions.contains("pixel_buffer_object") ) {
		mglGenBuffers = (GLGENBUFFERSARB)context->getProcAddress( "glGenBuffersARB" );
		mglBindBuffer = (GLBINDBUFFERARB)context->getProcAddress( "glBindBufferARB" );
		mglBufferData = (GLBUFFERDATAARB)context->getProcAddress( "glBufferDataARB" );
		mglMapBuffer = (GLMAPBUFFERARB)context->getProcAddress( "glMapBufferARB" );
		mglUnmapBuffer = (GLUNMAPBUFFERARB)context->getProcAddress( "glUnmapBufferARB" );
		hasPBO = mglGenBuffers && mglBindBuffer && mglBufferData && mglMapBuffer && mglUnmapBuffer;
	}
	else
		hasPBO = false;

	mglXSwapInterval = (GLXSWAPINTERVALSGI)context->getProcAddress( "glXSwapIntervalSGI" );
	hasVideoSync = mglXSwapInterval;
}



void GLFunc::drawRect( QRectF &rect )
{
	glBegin( GL_LINE_LOOP );
		glVertex3f( rect.x(), rect.y(), -5.);
		glVertex3f( rect.x()+rect.width(), rect.y(), -5.);
		glVertex3f( rect.x()+rect.width(), rect.y()+rect.height(), -5.);
		glVertex3f( rect.x(), rect.y()+rect.height(), -5.);
	glEnd ();
}



void GLFunc::drawHorizontalGradient( QRect &rect, const QColor &left, const QColor &right )
{
	float div=255.0;

	glBegin (GL_QUADS);
		glColor4f( left.red()/div, left.green()/div, left.blue()/div, left.alpha()/div );
		glVertex3f( rect.x(), rect.y(), -5.);
		glVertex3f( rect.x(), rect.y()+rect.height(), -5.);
		glColor4f( right.red()/div, right.green()/div, right.blue()/div, right.alpha()/div );
		glVertex3f( rect.x()+rect.width(), rect.y()+rect.height(), -5.);
		glVertex3f( rect.x()+rect.width(), rect.y(), -5.);
	glEnd ();
}



void GLFunc::drawVerticalGradient( QRect &rect, const QColor &top, const QColor &bottom )
{
	float div=255.0;

	glBegin (GL_QUADS);
		glColor4f( top.red()/div, top.green()/div, top.blue()/div, top.alpha()/div );
		glVertex3f( rect.x(), rect.y(), -5.);
		glVertex3f( rect.x()+rect.width(), rect.y(), -5.);
		glColor4f( bottom.red()/div, bottom.green()/div, bottom.blue()/div, bottom.alpha()/div );
		glVertex3f( rect.x()+rect.width(), rect.y()+rect.height(), -5.);
		glVertex3f( rect.x(), rect.y()+rect.height(), -5.);
	glEnd ();
}



void GLFunc::drawQuad( QRect &rect, const QColor &color )
{
	float div=255.0;
	glColor4f( color.red()/div, color.green()/div, color.blue()/div, color.alpha()/div );
	glBegin (GL_QUADS);
		glVertex3f( rect.x(), rect.y(), -5.);
		glVertex3f( rect.x()+rect.width(), rect.y(), -5.);
		glVertex3f( rect.x()+rect.width(), rect.y()+rect.height(), -5.);
		glVertex3f( rect.x(), rect.y()+rect.height(), -5.);
	glEnd ();
}



void GLFunc::drawQuadTexture( GLuint texture, QRect &rect, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1 )
{
	drawQuadTexture( texture, rect.x(), rect.y(), rect.width(), rect.height(), u, v, u1, v1 );
}



void GLFunc::drawQuadTexture( GLuint texture, QRectF &rect, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1 )
{
	drawQuadTexture( texture, rect.x(), rect.y(), rect.width(), rect.height(), u, v, u1, v1 );
}



void GLFunc::drawQuadTexture( GLuint texture, float x, float y, float w, float h, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1 )
{
	mglActiveTexture( GL_TEXTURE0 );
	glBindTexture( defaultTarget, texture );
	glBegin (GL_QUADS);
		glTexCoord2d( u, v );
		glVertex3f( x, y, -5.);
		glTexCoord2d( u1, v );
		glVertex3f( x+w, y, -5.);
		glTexCoord2d( u1, v1 );
		glVertex3f( x+w, y+h, -5.);
		glTexCoord2d( u, v1 );
		glVertex3f( x, y+h, -5.);
	glEnd ();
	glBindTexture( defaultTarget, 0 );
}



void GLFunc::textureFromData( GLuint texture, int width, int height, unsigned char *data, int &texW, int &texH )
{
	mglActiveTexture( GL_TEXTURE0 );
	glBindTexture( defaultTarget, texture );
	if ( hasTextureRect ) {
		texW = width;
		texH = height;
	}
	else {
		texW = 2; texH = 2;
		while ( texW<width ) texW <<= 1;
		while ( texH<height ) texH <<= 1;
	}
	glTexParameteri( defaultTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( defaultTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	if ( !hasTextureRect ) {
		glTexImage2D( defaultTarget, 0, 4, texW, texH, 0, GL_BGRA, GL_UNSIGNED_BYTE, NULL );
		glTexSubImage2D( defaultTarget, 0, 0, 0, width, height, GL_BGRA, GL_UNSIGNED_BYTE, data );
	}
	else
		glTexImage2D( defaultTarget, 0, 4, width, height, 0, GL_BGRA, GL_UNSIGNED_BYTE, data );
	glBindTexture( defaultTarget, 0 );
}
