/*
 * mgl.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef MGL_H
#define MGL_H

#include <QGLWidget>



class GLFunc
{

public:

	GLFunc( const QGLContext *ct );

	void init();

	void drawRect( QRectF &rect );
	void drawHorizontalGradient( QRect &rect, const QColor &left, const QColor &right );
	void drawVerticalGradient( QRect &rect, const QColor &top, const QColor &bottom );
	void drawQuad( QRect &rect, const QColor &color );
	void drawQuadTexture( GLuint texture, QRect &rect, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1 );
	void drawQuadTexture( GLuint texture, QRectF &rect, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1 );
	void drawQuadTexture( GLuint texture, float x, float y, float w, float h, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1 );
	void textureFromData( GLuint texture, int width, int height, unsigned char *data, int &texW, int &texH );

	bool doFragProg() { return hasFragProg; }
	bool doFragShader() { return hasFragShader; }
	bool doPBO() { return hasPBO; }
	bool doVideoSync() { return hasVideoSync; }
	bool doTextureRect() { return hasTextureRect; }

	typedef void (*GLACTIVETEXTUREARB) ( GLenum );
	GLACTIVETEXTUREARB mglActiveTexture;
	// FragProg
	typedef void (*GLGENPROGRAMSARB) ( GLsizei, GLuint* );
	typedef void (*GLBINDPROGRAMARB) ( GLenum, GLuint );
	typedef void (*GLPROGRAMSTRINGARB) ( GLenum, GLenum, GLsizei, const GLvoid* );
	GLGENPROGRAMSARB mglGenPrograms;
	GLBINDPROGRAMARB mglBindProgram;
	GLPROGRAMSTRINGARB mglProgramString;
	// FragShader
	typedef GLhandleARB (*GLCREATEPROGRAMOBJECTARB) ( void );
	typedef GLhandleARB (*GLCREATESHADEROBJECTARB) ( GLenum );
	typedef void (*GLSHADERSOURCEARB) ( GLhandleARB, GLsizei, const GLcharARB**, const GLint* );
	typedef void (*GLCOMPILESHADERARB) ( GLhandleARB );
	typedef void (*GLGETOBJECTPARAMETERIVARB) ( GLhandleARB, GLenum, GLint* );
	typedef void (*GLGETINFOLOGARB) ( GLhandleARB, GLsizei, GLsizei*, GLcharARB* );
	typedef void (*GLATTACHOBJECTARB) ( GLhandleARB, GLhandleARB );
	typedef void (*GLLINKPROGRAMARB) ( GLhandleARB );
	typedef void (*GLUSEPROGRAMOBJECTARB) ( GLhandleARB );
	typedef GLint (*GLGETUNIFORMLOCATIONARB) ( GLhandleARB, const GLcharARB* );
	typedef void (*GLUNIFORM1IARB) ( GLint, GLint );
	GLCREATEPROGRAMOBJECTARB mglCreateProgramObject;
	GLCREATESHADEROBJECTARB mglCreateShaderObject;
	GLSHADERSOURCEARB mglShaderSource;
	GLCOMPILESHADERARB mglCompileShader;
	GLGETOBJECTPARAMETERIVARB mglGetObjectParameteriv;
	GLGETINFOLOGARB mglGetInfoLog;
	GLATTACHOBJECTARB mglAttachObject;
	GLLINKPROGRAMARB mglLinkProgram;
	GLUSEPROGRAMOBJECTARB mglUseProgramObject;
	GLGETUNIFORMLOCATIONARB mglGetUniformLocation;
	GLUNIFORM1IARB mglUniform1i;
	// PBO
	typedef void (*GLGENBUFFERSARB) ( GLsizei, GLuint* );
	typedef void (*GLBINDBUFFERARB) ( GLenum, GLuint );
	typedef void (*GLBUFFERDATAARB) ( GLenum, GLsizeiptrARB, const GLvoid*, GLenum );
	typedef  GLvoid* (*GLMAPBUFFERARB) ( GLenum, GLenum );
	typedef GLboolean (*GLUNMAPBUFFERARB) ( GLenum );
	GLGENBUFFERSARB mglGenBuffers;
	GLBINDBUFFERARB mglBindBuffer;
	GLBUFFERDATAARB mglBufferData;
	GLMAPBUFFERARB mglMapBuffer;
	GLUNMAPBUFFERARB mglUnmapBuffer;
	// synctovblank
	typedef int (*GLXSWAPINTERVALSGI) ( int );
	GLXSWAPINTERVALSGI mglXSwapInterval;

private:

	bool hasFragProg;
	bool hasFragShader;
	bool hasPBO;
	bool hasVideoSync;
	bool hasTextureRect;
	GLenum defaultTarget;
	const QGLContext *context;
};

#endif // MGL_H
