/*
 * osdmenu.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "osdmenu.h"



OsdMenu::OsdMenu( OsdConfig *conf ) : OsdAnim( conf )
{
}



void OsdMenu::appendButton( OsdButton *btn )
{
	buttonList.append( btn );
	connect( btn, SIGNAL(wantRepaint()), this, SIGNAL(wantRepaint()) );
}



OsdMenu::~OsdMenu()
{
	for ( int i=0; i<buttonList.size(); ++i )
		delete buttonList.at(i);
}



void OsdMenu::mouseMove( QPoint &p )
{
	if ( isHidden() || isMoving() )
		return;

	if ( !inMe( p.x(), p.y() ) && !hasFocus )
		return;
	else if ( !inMe( p.x(), p.y() ) )
		hasFocus = false;
	else
		hasFocus = true;

	for ( int i=0; i<buttonList.size(); ++i ) {
		buttonList.at(i)->mouseMove( p );
	}
}



bool OsdMenu::mouseClicked( QMouseEvent *e )
{
	if ( isHidden() || isMoving() )
		return false;

	if ( !inMe( e->x(), e->y() ) )
		return false;
	for ( int i=0; i<buttonList.size(); ++i ) {
		if ( buttonList.at(i)->mouseClicked( e ) )
			return true;
	}
	return false;
}



void OsdMenu::paint()
{
	if ( isHidden() && !isMoving() )
		return;

	setTransform();

	config->func->drawQuad( rect, config->backgroundColor() );

	for ( int i=0; i<buttonList.size(); ++i ) {
		if ( buttonList.at(i)->x()>(rect.x()+rect.width()) )
			break;
		buttonList.at(i)->paint();
	}

	resetTransform();
}



void OsdMenu::resize( int w, int h )
{
	Q_UNUSED(h);
	QFontMetrics fm( config->normalFont() );
	QRect r;
	int tmp=0;
	rect.setWidth( w );
	for ( int i=0; i<buttonList.size(); ++i ) {
		r = fm.boundingRect( QRect(), Qt::AlignCenter, buttonList.at(i)->currentText() );
		buttonList.at(i)->setGeometry( rect.x()+MARGINSIZE+tmp, rect.y()+MARGINSIZE, r.width()+(2*MARGINSIZE), r.height() );
		tmp += r.width()+(2*MARGINSIZE)+MARGINSIZE;
	}
	rect.setHeight( r.height()+(2*MARGINSIZE) );
}
